'''
Collection Classes and Dimensional DataFrames
page 49-53 and python collection classes

data types python
bit <0/1> -> bytes <01000001> -> char [a-zA-Z0-9] -> string -> ... <values>

Collection of values -> independent collection of values
'Bob', 'Joe', 'Mary','Joe'
List indexed, mutatable, dups ['Bob', 'Joe', 'Mary','Joe',...]
Set !index, !sorted, !dups {'Joe','Mary','Bob'}.sort()
Tuple myFunc() immutable, (4111, '12/25', 524, 32765, True)
Dict/Map { key: value }

DataFrame dimensional data -> DEPENDENT values
1 Dimensional -> Series/Array -> col -> <value> pandas DataFrame
2 Dimensional -> row/col -> <value> 'what is the value in the Cell in Excel B3'
3 Dimensional -> pivot1 aka pivot table in Excel p1 -> row/col - <value>
4 Dimensional -> p2 p1 -> row/col - <value>
...
7 Dimensional -> p5,p4,p3,p2,p1 -> row/col - <value>
'''

import pandas as pd

contact = ['Bob','Joe','Mary','Joe','Beth']
age = [ 21, 35, 42, 15, 29]
zipcodes = [ '12542', '52144', '52365', '50214', '52145']

# Define our structured data from 2 independent lists
# Best Practices -> set of key:value pairs
data = {
    "contact": contact,
    "age": age,
    "zip codes": zipcodes
}

print(data)

# use the data set and create a dataFrame
# use the DataFrame class from pandas
df = pd.DataFrame(data)

print(df)

# Access specific data in dataframe
# .iloc index  best practice
# .loc location

row2 = df.iloc[2] # index 2 
print(row2)
row2l = df.loc[2]  # locations name row name 2
print(row2l)


# cols
# contact, age -> location/series name [contact] easier
# 0<contact>, 1<age> -> index .iloc[0] faster

series_contact = df['contact']
print(series_contact)

# aggregates
series_age = df['age']
print(series_age)
ave_age = series_age.mean()
min_age = series_age.min()
print(f"Average age of contacts is {ave_age}, and the youngest contact is {min_age} years old.")

# how to handle spaces in the series name
# back tick is the wrapper for the name
# example '`series space`'
# " ' ` ` ' "
series_zipcodes = df['zip codes']
print(series_zipcodes)

zipcodes50000 = df.query('`zip codes` >= "50000"')

print(zipcodes50000)

# head() tail() default top and bottom 5 rows dataframe
# head(3) param = 3 top 3 tail(2) bottom 2
top2 = df.head(2) # top 2
print(top2)

# info() information about the df object

print(df.info())

# adding a column to the end of the dataframe
df['ageIn10Years'] = df['age'] + 10
print(df)

# pip install openpyxl -> allow us to export pandas to excel

df.to_excel('contacts.xlsx', sheet_name='newcontacts', index=False)