# imports

import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
import openpyxl

# data set
filename = 'stocks.xlsx'
df = pd.read_excel(filename)

def mySeabornRelational():
    '''
    Relational Plots
    scatterplot -> correlation between two variables
    lineplot -> trend over time
    relplot -> type of figure that interfaces with scatter/line    
    '''
    # Create a figure with subplots for both scatter and line plots
    fig, (ax1, ax2) = plt.subplots(2,1, figsize=(12,10))

    # Scatter Plot  ax1
    sns.scatterplot(data=df, x='Open', y='Close', hue='High', size='Low', sizes=(20,200), ax=ax1)
    # 2 ways to set the outside values page 153
    # set() with attrs for title,xlabel,ylabel,...
    # set_title()
    ax1.set_title('Stock Open vs Close Price Scatter Plot')
    ax1.set(xlabel='Opening Price', ylabel='Closing Price')

    # Line Plot ax2
    sns.lineplot(data=df, x='Date', y='Close', ax=ax2)
    ax2.set_title('Stock Close over Time')
    ax2.tick_params(axis='x', rotation=45)

    plt.tight_layout()
    plt.savefig('seabornRealtionals.png', dpi=300, bbox_inches='tight')
    plt.close()

def mySeabornCategorical():
    '''
    bar -> mean and confidence intervals  ax4
    box -> show outliers, medians, quartiles (0,25,50,75,100) ax1
    violin -> shows distribution of data as a shape ax2
    swarm -> all data points with no overlap plot ax3
    '''
    # 2,2
    fig, ((ax1,ax2), (ax3,ax4)) = plt.subplots(2,2, figsize=(16,12)) #8,6

    # Categorize our stock closing prices into 4 groups and add that as a new series/col in our dataframe Date,Open,High,Low,Close,[Price_Range]
    df['Price_Range'] = pd.qcut(df['Close'], q=4, labels=['Low', 'Medium','High','Very High'])

    print(df)

    # Boxplot ax1
    sns.boxplot(data=df, x='Price_Range', y='Close', ax=ax1)
    ax1.set_title('Volume Distribution by Price Range')

    # Violin Shape ax2
    sns.violinplot(data=df, x='Price_Range', y='High', ax=ax2)
    ax1.set_title('High Price Distribution by Price Range')

    # Swarm -> no overlaping visually ax3
    sns.swarmplot(data=df.sample(100), x='Price_Range', y='Low', ax=ax3)
    # sample() is a set of sample data, sets have no dups
    ax3.set_title('Low Prices by Range (Sample Data)')

    # Bar Plot ax4
    sns.barplot(data=df, x='Price_Range', y='Close', ax=ax4)
    ax4.set_title('Average Close by Price Range')

    plt.tight_layout()
    plt.savefig('seabornCategoricals.png', dpi=300, bbox_inches='tight')
    plt.close()

def mySeabornDistribution():
    '''
    Figure -> shapes of data distribution
    histplot -> frequency distribution (with a single density)
    --> density attr in histplot to show density as sub of frequency
    --> kde=True
    kdeplot -> density distribution (with multiple densities)    
    '''

    # Create a figure with 2 subplots ax1 host ax2 density
    fig, (ax1,ax2) = plt.subplots(1,2, figsize=(16,6))

    # Histogram with Single Density Variable
    sns.histplot(data=df, x='Close', kde=True, ax=ax1) # kde 'Close'
    ax1.set_title('Distribution of Closing Prices')

    # Density Plot with either one or more density variables
    sns.kdeplot(data=df[['Open','High','Low','Close']], ax=ax2)
    ax2.set_title('Density Distribution of All Prices')
    ax2.set_facecolor('#bb0000')

    plt.tight_layout()
    plt.savefig('seabornDistributions.png', dpi=300, bbox_inches='tight', facecolor='#00bb00', edgecolor='red', pad_inches=0.2)
    plt.close()

# call the function to plot
mySeabornDistribution()

'''
Additional Seaborn Customizations

# Styling -> sets a global attr for all plots running module
sns.set_style(param1)

# Set axis limits aka the ranges of the axis
ax1.set_xlim()
ax1.set_ylim()

# Customizing ticks tick_params()
axis='x'
rotation=int
colors='darkblue' also be #RRGGBB

# Add a background color to the plot
ax.set_facecolor('#ff0000')
'''
